using Microsoft.Extensions.Configuration;
using System.Text.Json;
using System.Text.Json.Serialization;

namespace File_Folder_Helper.Models.Binder;

public class AppSettings
{

    public string? Company { get; set; }
    public string? DefaultNoteType { get; set; }
    public string[]? ExcludeDirectoryNames { get; set; }
    public string[]? ExcludeSchemes { get; set; }
    public string? PersonBirthdayFormat { get; set; }
    public string? PersonCharacters { get; set; }
    public string? PersonTitleFilters { get; set; }
    public string? WorkingDirectoryName { get; set; }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, BinderAppSettingsSourceGenerationContext.Default.AppSettings);
        return result;
    }

    private static Models.AppSettings Get(AppSettings? appSettings)
    {
        Models.AppSettings result;
        if (appSettings?.Company is null) throw new NullReferenceException(nameof(appSettings.Company));
        if (appSettings?.DefaultNoteType is null) throw new NullReferenceException(nameof(appSettings.DefaultNoteType));
        if (appSettings?.ExcludeDirectoryNames is null) throw new NullReferenceException(nameof(appSettings.ExcludeDirectoryNames));
        if (appSettings?.ExcludeSchemes is null) throw new NullReferenceException(nameof(appSettings.ExcludeSchemes));
        if (appSettings?.PersonBirthdayFormat is null) throw new NullReferenceException(nameof(appSettings.PersonBirthdayFormat));
        if (appSettings?.PersonCharacters is null) throw new NullReferenceException(nameof(appSettings.PersonCharacters));
        if (appSettings?.PersonTitleFilters is null) throw new NullReferenceException(nameof(appSettings.PersonTitleFilters));
        if (appSettings?.WorkingDirectoryName is null) throw new NullReferenceException(nameof(appSettings.WorkingDirectoryName));
        result = new(
            appSettings.Company,
            appSettings.DefaultNoteType,
            appSettings.ExcludeDirectoryNames,
            appSettings.ExcludeSchemes,
            appSettings.PersonBirthdayFormat,
            appSettings.PersonCharacters.ToArray(),
            appSettings.PersonTitleFilters.ToArray(),
            appSettings.WorkingDirectoryName
        );
        return result;
    }

    public static Models.AppSettings Get(IConfigurationRoot configurationRoot)
    {
        Models.AppSettings result;
#pragma warning disable IL3050, IL2026
        AppSettings? appSettings = configurationRoot.Get<AppSettings>();
#pragma warning restore IL3050, IL2026
        if (appSettings?.Company is null)
        {
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers)
            {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                if (!physicalFileProvider.Root.Contains("UserSecrets"))
                    continue;
                throw new NotSupportedException(physicalFileProvider.Root);
            }
        }
        result = Get(appSettings);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(AppSettings))]
internal partial class BinderAppSettingsSourceGenerationContext : JsonSerializerContext
{
}